@@ -93,6 +93,35 @@ flags.binary;
  */
 Document.prototype.currentScript;
 
+/**
+ * Don't minify Math.*
+ */
+/**
+ * @suppress {duplicate}
+ */
+var Math = {};
+Math.abs = function() {};
+Math.cos = function() {};
+Math.sin = function() {};
+Math.tan = function() {};
+Math.acos = function() {};
+Math.asin = function() {};
+Math.atan = function() {};
+Math.atan2 = function() {};
+Math.exp = function() {};
+Math.log = function() {};
+Math.sqrt = function() {};
+Math.ceil = function() {};
+Math.floor = function() {};
+Math.pow = function() {};
+Math.imul = function() {};
+Math.fround = function() {};
+Math.round = function() {};
+Math.min = function() {};
+Math.max = function() {};
+Math.clz32 = function() {};
+Math.trunc = function() {};
+
 /**
  * SIMD.js support (not in upstream closure yet).
  */
@@ -1257,14 +1286,6 @@ var _emscripten_glDisableVertexAttribArray;
  */
 var _emscripten_glVertexAttribPointer;
 
-/**
- * @suppress {duplicate, undefinedVars}
- */
-var _glDrawArrays;
-/**
- * @suppress {duplicate, undefinedVars}
- */
-var _glDrawElements;
 /**
  * @suppress {duplicate, undefinedVars}
  */
@@ -1294,7 +1315,7 @@ var _glutPostRedisplay = function() {};
 var emscripten_source_map;
 
 /**
- * @typy {Worker}
+ * @type {Worker}
  */
 var worker;
 /**
@@ -1335,3 +1356,9 @@ var __setLetterbox = function(element, topBottom, leftRight) {};
  */
 var onmessage = function(message) {};
 
+// Fetch.js/Fetch Worker
+
+/**
+ * @suppress {undefinedVars}
+ */
+var ENVIRONMENT_IS_FETCH_WORKER;
