@@ -1,3 +1,8 @@
+// Copyright 2012 The Emscripten Authors.  All rights reserved.
+// Emscripten is available under two separate licenses, the MIT license and the
+// University of Illinois/NCSA Open Source License.  Both these licenses can be
+// found in the LICENSE file.
+
 /*global LibraryManager, mergeInto*/
 
 /*global Module, asm*/
@@ -11,7 +16,7 @@
 
 // -- jshint doesn't understand library syntax, so we need to specifically tell it about the symbols we define
 /*global typeDependencies, flushPendingDeletes, getTypeName, getBasestPointer, throwBindingError, UnboundTypeError, _embind_repr, registeredInstances, registeredTypes, getShiftFromSize*/
-/*global ensureOverloadTable, requireFunction, awaitingDependencies, makeLegalFunctionName, embind_charCodes:true, registerType, createNamedFunction, RegisteredPointer, throwInternalError*/
+/*global ensureOverloadTable, embind__requireFunction, awaitingDependencies, makeLegalFunctionName, embind_charCodes:true, registerType, createNamedFunction, RegisteredPointer, throwInternalError*/
 /*global simpleReadValueFromPointer, floatReadValueFromPointer, integerReadValueFromPointer, enumReadValueFromPointer, replacePublicSymbol, craftInvokerFunction, tupleRegistrations*/
 /*global ClassHandle, makeClassHandle, structRegistrations, whenDependentTypesAreResolved, BindingError, deletionQueue, delayFunction:true, upcastPointer*/
 /*global exposePublicSymbol, heap32VectorToArray, new_, RegisteredPointer_getPointee, RegisteredPointer_destructor, RegisteredPointer_deleteObject, char_0, char_9*/
@@ -45,10 +50,13 @@ var LibraryEmbind = {
     Module['flushPendingDeletes'] = flushPendingDeletes;
     Module['setDelayFunction'] = setDelayFunction;
 #if IN_TEST_HARNESS
-#if NO_DYNAMIC_EXECUTION
+#if DYNAMIC_EXECUTION
     // Without dynamic execution, dynamically created functions will have no
     // names. This lets the test suite know that.
-    Module['NO_DYNAMIC_EXECUTION'] = true;
+    Module['DYNAMIC_EXECUTION'] = true;
+#endif
+#if EMBIND_STD_STRING_IS_UTF8
+    Module['EMBIND_STD_STRING_IS_UTF8'] = true;
 #endif
 #endif
   },
@@ -186,7 +194,7 @@ var LibraryEmbind = {
   $createNamedFunction__deps: ['$makeLegalFunctionName'],
   $createNamedFunction: function(name, body) {
     name = makeLegalFunctionName(name);
-#if NO_DYNAMIC_EXECUTION
+#if DYNAMIC_EXECUTION == 0
     return function() {
       "use strict";
       return body.apply(this, arguments);
@@ -538,11 +546,11 @@ var LibraryEmbind = {
     }
 
     var shift = getShiftFromSize(size);
-    
+
     var fromWireType = function(value) {
         return value;
     };
-    
+
     if (minRange === 0) {
         var bitshift = 32 - 8*size;
         fromWireType = function(value) {
@@ -608,53 +616,103 @@ var LibraryEmbind = {
     '$simpleReadValueFromPointer', '$throwBindingError'],
   _embind_register_std_string: function(rawType, name) {
     name = readLatin1String(name);
+    var stdStringIsUTF8
+#if EMBIND_STD_STRING_IS_UTF8
+    //process only std::string bindings with UTF8 support, in contrast to e.g. std::basic_string<unsigned char>
+    = (name === "std::string");
+#else
+    = false;
+#endif
+
     registerType(rawType, {
         name: name,
         'fromWireType': function(value) {
             var length = HEAPU32[value >> 2];
-            var a = new Array(length);
-            for (var i = 0; i < length; ++i) {
-                a[i] = String.fromCharCode(HEAPU8[value + 4 + i]);
+
+            var str;
+            if(stdStringIsUTF8) {
+                //ensure null termination at one-past-end byte if not present yet
+                var endChar = HEAPU8[value + 4 + length];
+                var endCharSwap = 0;
+                if(endChar != 0)
+                {
+                  endCharSwap = endChar;
+                  HEAPU8[value + 4 + length] = 0;
+                }
+
+                var decodeStartPtr = value + 4;
+                //looping here to support possible embedded '0' bytes
+                for (var i = 0; i <= length; ++i) {
+                  var currentBytePtr = value + 4 + i;
+                  if(HEAPU8[currentBytePtr] == 0)
+                  {
+                    var stringSegment = UTF8ToString(decodeStartPtr);
+                    if(str === undefined)
+                      str = stringSegment;
+                    else
+                    {
+                      str += String.fromCharCode(0);
+                      str += stringSegment;
+                    }
+                    decodeStartPtr = currentBytePtr + 1;
+                  }
+                }
+
+                if(endCharSwap != 0)
+                  HEAPU8[value + 4 + length] = endCharSwap;
+            } else {
+                var a = new Array(length);
+                for (var i = 0; i < length; ++i) {
+                    a[i] = String.fromCharCode(HEAPU8[value + 4 + i]);
+                }
+                str = a.join('');
             }
+
             _free(value);
-            return a.join('');
+            
+            return str;
         },
         'toWireType': function(destructors, value) {
             if (value instanceof ArrayBuffer) {
                 value = new Uint8Array(value);
             }
+            
+            var getLength;
+            var valueIsOfTypeString = (typeof value === 'string');
 
-            function getTAElement(ta, index) {
-                return ta[index];
-            }
-            function getStringElement(string, index) {
-                return string.charCodeAt(index);
-            }
-            var getElement;
-            if (value instanceof Uint8Array) {
-                getElement = getTAElement;
-            } else if (value instanceof Uint8ClampedArray) {
-                getElement = getTAElement;
-            } else if (value instanceof Int8Array) {
-                getElement = getTAElement;
-            } else if (typeof value === 'string') {
-                getElement = getStringElement;
-            } else {
+            if (!(valueIsOfTypeString || value instanceof Uint8Array || value instanceof Uint8ClampedArray || value instanceof Int8Array)) {
                 throwBindingError('Cannot pass non-string to std::string');
             }
-
-            // assumes 4-byte alignment
-            var length = value.length;
-            var ptr = _malloc(4 + length);
-            HEAPU32[ptr >> 2] = length;
-            for (var i = 0; i < length; ++i) {
-                var charCode = getElement(value, i);
-                if (charCode > 255) {
-                    _free(ptr);
-                    throwBindingError('String has UTF-16 code units that do not fit in 8 bits');
-                }
-                HEAPU8[ptr + 4 + i] = charCode;
+            if (stdStringIsUTF8 && valueIsOfTypeString) {
+                getLength = function() {return lengthBytesUTF8(value);};
+            } else {
+                getLength = function() {return value.length;};
             }
+            
+            // assumes 4-byte alignment
+            var length = getLength();
+            var ptr = _malloc(4 + length + 1);
+            HEAPU32[ptr >> 2] = length;
+
+            if (stdStringIsUTF8 && valueIsOfTypeString) {
+                stringToUTF8(value, ptr + 4, length + 1);
+            } else {
+                if(valueIsOfTypeString) {
+                    for (var i = 0; i < length; ++i) {
+                        var charCode = value.charCodeAt(i);
+                        if (charCode > 255) {
+                            _free(ptr);
+                            throwBindingError('String has UTF-16 code units that do not fit in 8 bits');
+                        }
+                        HEAPU8[ptr + 4 + i] = charCode;
+                    }
+                } else {
+                    for (var i = 0; i < length; ++i) {
+                        HEAPU8[ptr + 4 + i] = value[i];
+                    }
+                }
+            }
+
             if (destructors !== null) {
                 destructors.push(_free, ptr);
             }
@@ -789,9 +847,9 @@ var LibraryEmbind = {
     if (!(constructor instanceof Function)) {
         throw new TypeError('new_ called with constructor type ' + typeof(constructor) + " which is not a function");
     }
-#if NO_DYNAMIC_EXECUTION
+#if DYNAMIC_EXECUTION == 0
     if (constructor === Function) {
-      throw new Error('new_ cannot create a new Function with NO_DYNAMIC_EXECUTION.');
+      throw new Error('new_ cannot create a new Function with DYNAMIC_EXECUTION == 0.');
     }
 #endif
 
@@ -855,7 +913,7 @@ var LibraryEmbind = {
 
     var returns = (argTypes[0].name !== "void");
 
-#if NO_DYNAMIC_EXECUTION
+#if DYNAMIC_EXECUTION == 0
     var argsWired = new Array(argCount - 2);
     return function() {
       if (arguments.length !== argCount - 2) {
@@ -979,12 +1037,12 @@ var LibraryEmbind = {
 #endif
   },
 
-  $requireFunction__deps: ['$readLatin1String', '$throwBindingError'],
-  $requireFunction: function(signature, rawFunction) {
+  $embind__requireFunction__deps: ['$readLatin1String', '$throwBindingError'],
+  $embind__requireFunction: function(signature, rawFunction) {
     signature = readLatin1String(signature);
 
     function makeDynCaller(dynCall) {
-#if NO_DYNAMIC_EXECUTION
+#if DYNAMIC_EXECUTION == 0
       return function() {
           var args = new Array(arguments.length + 1);
           args[0] = rawFunction;
@@ -1023,13 +1081,13 @@ var LibraryEmbind = {
         // This has three main penalties:
         // - dynCall is another function call in the path from JavaScript to C++.
         // - JITs may not predict through the function table indirection at runtime.
-        var dc = Module["asm"]['dynCall_' + signature];
+        var dc = Module['dynCall_' + signature];
         if (dc === undefined) {
             // We will always enter this branch if the signature
             // contains 'f' and PRECISE_F32 is not enabled.
             //
             // Try again, replacing 'f' with 'd'.
-            dc = Module["asm"]['dynCall_' + signature.replace(/f/g, 'd')];
+            dc = Module['dynCall_' + signature.replace(/f/g, 'd')];
             if (dc === undefined) {
                 throwBindingError("No dynCall invoker for signature: " + signature);
             }
@@ -1045,13 +1103,13 @@ var LibraryEmbind = {
 
   _embind_register_function__deps: [
     '$craftInvokerFunction', '$exposePublicSymbol', '$heap32VectorToArray',
-    '$readLatin1String', '$replacePublicSymbol', '$requireFunction',
+    '$readLatin1String', '$replacePublicSymbol', '$embind__requireFunction',
     '$throwUnboundTypeError', '$whenDependentTypesAreResolved'],
   _embind_register_function: function(name, argCount, rawArgTypesAddr, signature, rawInvoker, fn) {
     var argTypes = heap32VectorToArray(argCount, rawArgTypesAddr);
     name = readLatin1String(name);
-    
-    rawInvoker = requireFunction(signature, rawInvoker);
+
+    rawInvoker = embind__requireFunction(signature, rawInvoker);
 
     exposePublicSymbol(name, function() {
         throwUnboundTypeError('Cannot call ' + name + ' due to unbound types', argTypes);
@@ -1067,7 +1125,7 @@ var LibraryEmbind = {
   $tupleRegistrations: {},
 
   _embind_register_value_array__deps: [
-    '$tupleRegistrations', '$readLatin1String', '$requireFunction'],
+    '$tupleRegistrations', '$readLatin1String', '$embind__requireFunction'],
   _embind_register_value_array: function(
     rawType,
     name,
@@ -1078,14 +1136,14 @@ var LibraryEmbind = {
   ) {
     tupleRegistrations[rawType] = {
         name: readLatin1String(name),
-        rawConstructor: requireFunction(constructorSignature, rawConstructor),
-        rawDestructor: requireFunction(destructorSignature, rawDestructor),
+        rawConstructor: embind__requireFunction(constructorSignature, rawConstructor),
+        rawDestructor: embind__requireFunction(destructorSignature, rawDestructor),
         elements: [],
     };
   },
 
   _embind_register_value_array_element__deps: [
-    '$tupleRegistrations', '$requireFunction'],
+    '$tupleRegistrations', '$embind__requireFunction'],
   _embind_register_value_array_element: function(
     rawTupleType,
     getterReturnType,
@@ -1099,10 +1157,10 @@ var LibraryEmbind = {
   ) {
     tupleRegistrations[rawTupleType].elements.push({
         getterReturnType: getterReturnType,
-        getter: requireFunction(getterSignature, getter),
+        getter: embind__requireFunction(getterSignature, getter),
         getterContext: getterContext,
         setterArgumentType: setterArgumentType,
-        setter: requireFunction(setterSignature, setter),
+        setter: embind__requireFunction(setterSignature, setter),
         setterContext: setterContext,
     });
   },
@@ -1172,7 +1230,7 @@ var LibraryEmbind = {
   $structRegistrations: {},
 
   _embind_register_value_object__deps: [
-    '$structRegistrations', '$readLatin1String', '$requireFunction'],
+    '$structRegistrations', '$readLatin1String', '$embind__requireFunction'],
   _embind_register_value_object: function(
     rawType,
     name,
@@ -1183,14 +1241,14 @@ var LibraryEmbind = {
   ) {
     structRegistrations[rawType] = {
         name: readLatin1String(name),
-        rawConstructor: requireFunction(constructorSignature, rawConstructor),
-        rawDestructor: requireFunction(destructorSignature, rawDestructor),
+        rawConstructor: embind__requireFunction(constructorSignature, rawConstructor),
+        rawDestructor: embind__requireFunction(destructorSignature, rawDestructor),
         fields: [],
     };
   },
 
   _embind_register_value_object_field__deps: [
-    '$structRegistrations', '$readLatin1String', '$requireFunction'],
+    '$structRegistrations', '$readLatin1String', '$embind__requireFunction'],
   _embind_register_value_object_field: function(
     structType,
     fieldName,
@@ -1206,10 +1264,10 @@ var LibraryEmbind = {
     structRegistrations[structType].fields.push({
         fieldName: readLatin1String(fieldName),
         getterReturnType: getterReturnType,
-        getter: requireFunction(getterSignature, getter),
+        getter: embind__requireFunction(getterSignature, getter),
         getterContext: getterContext,
         setterArgumentType: setterArgumentType,
-        setter: requireFunction(setterSignature, setter),
+        setter: embind__requireFunction(setterSignature, setter),
         setterContext: setterContext,
     });
   },
@@ -1285,13 +1343,14 @@ var LibraryEmbind = {
 
   $genericPointerToWireType__deps: ['$throwBindingError', '$upcastPointer'],
   $genericPointerToWireType: function(destructors, handle) {
+    var ptr;
     if (handle === null) {
         if (this.isReference) {
             throwBindingError('null is not a valid ' + this.name);
         }
 
         if (this.isSmartPointer) {
-            var ptr = this.rawConstructor();
+            ptr = this.rawConstructor();
             if (destructors !== null) {
                 destructors.push(this.rawDestructor, ptr);
             }
@@ -1311,7 +1370,7 @@ var LibraryEmbind = {
         throwBindingError('Cannot convert argument of type ' + (handle.$$.smartPtrType ? handle.$$.smartPtrType.name : handle.$$.ptrType.name) + ' to parameter type ' + this.name);
     }
     var handleClass = handle.$$.ptrType.registeredClass;
-    var ptr = upcastPointer(handle.$$.ptr, handleClass, this.registeredClass);
+    ptr = upcastPointer(handle.$$.ptr, handleClass, this.registeredClass);
 
     if (this.isSmartPointer) {
         // TODO: this is not strictly true
@@ -1769,7 +1828,7 @@ var LibraryEmbind = {
     '$registeredPointers', '$exposePublicSymbol',
     '$makeLegalFunctionName', '$readLatin1String',
     '$RegisteredClass', '$RegisteredPointer', '$replacePublicSymbol',
-    '$requireFunction', '$throwUnboundTypeError',
+    '$embind__requireFunction', '$throwUnboundTypeError',
     '$whenDependentTypesAreResolved'],
   _embind_register_class: function(
     rawType,
@@ -1787,14 +1846,14 @@ var LibraryEmbind = {
     rawDestructor
   ) {
     name = readLatin1String(name);
-    getActualType = requireFunction(getActualTypeSignature, getActualType);
+    getActualType = embind__requireFunction(getActualTypeSignature, getActualType);
     if (upcast) {
-        upcast = requireFunction(upcastSignature, upcast);
+        upcast = embind__requireFunction(upcastSignature, upcast);
     }
     if (downcast) {
-        downcast = requireFunction(downcastSignature, downcast);
+        downcast = embind__requireFunction(downcastSignature, downcast);
     }
-    rawDestructor = requireFunction(destructorSignature, rawDestructor);
+    rawDestructor = embind__requireFunction(destructorSignature, rawDestructor);
     var legalFunctionName = makeLegalFunctionName(name);
 
     exposePublicSymbol(legalFunctionName, function() {
@@ -1881,7 +1940,7 @@ var LibraryEmbind = {
   },
 
   _embind_register_class_constructor__deps: [
-    '$heap32VectorToArray', '$requireFunction', '$runDestructors',
+    '$heap32VectorToArray', '$embind__requireFunction', '$runDestructors',
     '$throwBindingError', '$whenDependentTypesAreResolved'],
   _embind_register_class_constructor: function(
     rawClassType,
@@ -1892,7 +1951,7 @@ var LibraryEmbind = {
     rawConstructor
   ) {
     var rawArgTypes = heap32VectorToArray(argCount, rawArgTypesAddr);
-    invoker = requireFunction(invokerSignature, invoker);
+    invoker = embind__requireFunction(invokerSignature, invoker);
 
     whenDependentTypesAreResolved([], [rawClassType], function(classType) {
         classType = classType[0];
@@ -1979,7 +2038,7 @@ var LibraryEmbind = {
 
   _embind_register_class_function__deps: [
     '$craftInvokerFunction', '$heap32VectorToArray', '$readLatin1String',
-    '$requireFunction', '$throwUnboundTypeError',
+    '$embind__requireFunction', '$throwUnboundTypeError',
     '$whenDependentTypesAreResolved'],
   _embind_register_class_function: function(
     rawClassType,
@@ -1993,7 +2052,7 @@ var LibraryEmbind = {
   ) {
     var rawArgTypes = heap32VectorToArray(argCount, rawArgTypesAddr);
     methodName = readLatin1String(methodName);
-    rawInvoker = requireFunction(invokerSignature, rawInvoker);
+    rawInvoker = embind__requireFunction(invokerSignature, rawInvoker);
 
     whenDependentTypesAreResolved([], [rawClassType], function(classType) {
         classType = classType[0];
@@ -2041,7 +2100,7 @@ var LibraryEmbind = {
   },
 
   _embind_register_class_property__deps: [
-    '$readLatin1String', '$requireFunction', '$runDestructors',
+    '$readLatin1String', '$embind__requireFunction', '$runDestructors',
     '$throwBindingError', '$throwUnboundTypeError',
     '$whenDependentTypesAreResolved', '$validateThis'],
   _embind_register_class_property: function(
@@ -2057,7 +2116,7 @@ var LibraryEmbind = {
     setterContext
   ) {
     fieldName = readLatin1String(fieldName);
-    getter = requireFunction(getterSignature, getter);
+    getter = embind__requireFunction(getterSignature, getter);
 
     whenDependentTypesAreResolved([], [classType], function(classType) {
         classType = classType[0];
@@ -2095,7 +2154,7 @@ var LibraryEmbind = {
             };
 
             if (setter) {
-                setter = requireFunction(setterSignature, setter);
+                setter = embind__requireFunction(setterSignature, setter);
                 var setterArgumentType = types[1];
                 desc.set = function(v) {
                     var ptr = validateThis(this, classType, humanName + ' setter');
@@ -2115,7 +2174,7 @@ var LibraryEmbind = {
 
   _embind_register_class_class_function__deps: [
     '$craftInvokerFunction', '$ensureOverloadTable', '$heap32VectorToArray',
-    '$readLatin1String', '$requireFunction', '$throwUnboundTypeError',
+    '$readLatin1String', '$embind__requireFunction', '$throwUnboundTypeError',
     '$whenDependentTypesAreResolved'],
   _embind_register_class_class_function: function(
     rawClassType,
@@ -2128,7 +2187,7 @@ var LibraryEmbind = {
   ) {
     var rawArgTypes = heap32VectorToArray(argCount, rawArgTypesAddr);
     methodName = readLatin1String(methodName);
-    rawInvoker = requireFunction(invokerSignature, rawInvoker);
+    rawInvoker = embind__requireFunction(invokerSignature, rawInvoker);
     whenDependentTypesAreResolved([], [rawClassType], function(classType) {
         classType = classType[0];
         var humanName = classType.name + '.' + methodName;
@@ -2154,6 +2213,7 @@ var LibraryEmbind = {
             var invokerArgsArray = [argTypes[0] /* return value */, null /* no class 'this'*/].concat(argTypes.slice(1) /* actual params */);
             var func = craftInvokerFunction(humanName, invokerArgsArray, null /* no class 'this'*/, rawInvoker, fn);
             if (undefined === proto[methodName].overloadTable) {
+                func.argCount = argCount-1;
                 proto[methodName] = func;
             } else {
                 proto[methodName].overloadTable[argCount-1] = func;
@@ -2165,7 +2225,7 @@ var LibraryEmbind = {
   },
 
   _embind_register_class_class_property__deps: [
-    '$readLatin1String', '$requireFunction', '$runDestructors',
+    '$readLatin1String', '$embind__requireFunction', '$runDestructors',
     '$throwBindingError', '$throwUnboundTypeError',
     '$whenDependentTypesAreResolved', '$validateThis'],
   _embind_register_class_class_property: function(
@@ -2179,21 +2239,21 @@ var LibraryEmbind = {
     setter
   ) {
     fieldName = readLatin1String(fieldName);
-    getter = requireFunction(getterSignature, getter);
+    getter = embind__requireFunction(getterSignature, getter);
 
     whenDependentTypesAreResolved([], [rawClassType], function(classType) {
         classType = classType[0];
         var humanName = classType.name + '.' + fieldName;
         var desc = {
             get: function() {
-                throwUnboundTypeError('Cannot access ' + humanName + ' due to unbound types', [getterReturnType, setterArgumentType]);
+                throwUnboundTypeError('Cannot access ' + humanName + ' due to unbound types', [rawFieldType]);
             },
             enumerable: true,
             configurable: true
         };
         if (setter) {
             desc.set = function() {
-                throwUnboundTypeError('Cannot access ' + humanName + ' due to unbound types', [getterReturnType, setterArgumentType]);
+                throwUnboundTypeError('Cannot access ' + humanName + ' due to unbound types', [rawFieldType]);
             };
         } else {
             desc.set = function(v) {
@@ -2213,7 +2273,7 @@ var LibraryEmbind = {
             };
 
             if (setter) {
-                setter = requireFunction(setterSignature, setter);
+                setter = embind__requireFunction(setterSignature, setter);
                 desc.set = function(v) {
                     var destructors = [];
                     setter(rawFieldPtr, fieldType['toWireType'](destructors, v));
@@ -2310,7 +2370,7 @@ var LibraryEmbind = {
     }
   },
 
-  _embind_register_smart_ptr__deps: ['$RegisteredPointer', '$requireFunction', '$whenDependentTypesAreResolved'],
+  _embind_register_smart_ptr__deps: ['$RegisteredPointer', '$embind__requireFunction', '$whenDependentTypesAreResolved'],
   _embind_register_smart_ptr: function(
     rawType,
     rawPointeeType,
@@ -2326,10 +2386,10 @@ var LibraryEmbind = {
     rawDestructor
   ) {
     name = readLatin1String(name);
-    rawGetPointee = requireFunction(getPointeeSignature, rawGetPointee);
-    rawConstructor = requireFunction(constructorSignature, rawConstructor);
-    rawShare = requireFunction(shareSignature, rawShare);
-    rawDestructor = requireFunction(destructorSignature, rawDestructor);
+    rawGetPointee = embind__requireFunction(getPointeeSignature, rawGetPointee);
+    rawConstructor = embind__requireFunction(constructorSignature, rawConstructor);
+    rawShare = embind__requireFunction(shareSignature, rawShare);
+    rawDestructor = embind__requireFunction(destructorSignature, rawDestructor);
 
     whenDependentTypesAreResolved([rawType], [rawPointeeType], function(pointeeType) {
         pointeeType = pointeeType[0];
