@@ -1,8 +1,21 @@
+# Copyright 2012 The Emscripten Authors.  All rights reserved.
+# Emscripten is available under two separate licenses, the MIT license and the
+# University of Illinois/NCSA Open Source License.  Both these licenses can be
+# found in the LICENSE file.
 
 from __future__ import print_function
-import os, sys, subprocess, multiprocessing, re, string, json, shutil, logging
+import os
+import sys
+import subprocess
+import multiprocessing
+import re
+import string
+import json
+import shutil
+import logging
 
-sys.path.insert(1, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
+__rootpath__ = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
+sys.path.insert(1, __rootpath__)
 
 from tools.toolchain_profiler import ToolchainProfiler
 if __name__ == '__main__':
@@ -17,7 +30,6 @@ except ImportError:
 configuration = shared.configuration
 temp_files = configuration.get_temp_files()
 
-__rootpath__ = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
 def path_from_root(*pathelems):
   return os.path.join(__rootpath__, *pathelems)
 
@@ -33,9 +45,9 @@ WINDOWS = sys.platform.startswith('win')
 
 DEBUG = os.environ.get('EMCC_DEBUG')
 
-func_sig = re.compile('function ([_\w$]+)\(')
-func_sig_json = re.compile('\["defun", ?"([_\w$]+)",')
-import_sig = re.compile('(var|const) ([_\w$]+ *=[^;]+);')
+func_sig = re.compile(r'function ([_\w$]+)\(')
+func_sig_json = re.compile(r'\["defun", ?"([_\w$]+)",')
+import_sig = re.compile(r'(var|const) ([_\w$]+ *=[^;]+);')
 
 NATIVE_OPTIMIZER = os.environ.get('EMCC_NATIVE_OPTIMIZER') or '2' # use optimized native optimizer by default, unless disabled by EMCC_NATIVE_OPTIMIZER=0 in the env
 
@@ -56,18 +68,16 @@ def split_funcs(js, just_split=False):
   return funcs
 
 def get_native_optimizer():
-  if os.environ.get('EMCC_FAST_COMPILER') == '0':
-    logging.critical('Non-fastcomp compiler is no longer available, please use fastcomp or an older version of emscripten')
-    sys.exit(1)
-
-  # Allow users to override the location of the optimizer executable by setting an environment variable EMSCRIPTEN_NATIVE_OPTIMIZER=/path/to/optimizer(.exe)
-  opt = os.environ.get('EMSCRIPTEN_NATIVE_OPTIMIZER', '')
-  if len(opt):
+  # Allow users to override the location of the optimizer executable by setting
+  # an environment variable EMSCRIPTEN_NATIVE_OPTIMIZER=/path/to/optimizer(.exe)
+  opt = os.environ.get('EMSCRIPTEN_NATIVE_OPTIMIZER')
+  if opt:
     logging.debug('env forcing native optimizer at ' + opt)
     return opt
-  # Also, allow specifying the location of the optimizer in .emscripten configuration file under EMSCRIPTEN_NATIVE_OPTIMIZER='/path/to/optimizer'
-  opt = getattr(shared, 'EMSCRIPTEN_NATIVE_OPTIMIZER', '')
-  if len(opt):
+  # Also, allow specifying the location of the optimizer in .emscripten
+  # configuration file under EMSCRIPTEN_NATIVE_OPTIMIZER='/path/to/optimizer'
+  opt = shared.EMSCRIPTEN_NATIVE_OPTIMIZER
+  if opt:
     logging.debug('config forcing native optimizer at ' + opt)
     return opt
 
@@ -231,11 +241,10 @@ class Minifier(object):
       self.globs = []
 
     with temp_files.get_file('.minifyglobals.js') as temp_file:
-      f = open(temp_file, 'w')
-      f.write(shell)
-      f.write('\n')
-      f.write('// EXTRA_INFO:' + json.dumps(self.serialize()))
-      f.close()
+      with open(temp_file, 'w') as f:
+        f.write(shell)
+        f.write('\n')
+        f.write('// EXTRA_INFO:' + json.dumps(self.serialize()))
 
       output = shared.run_process(self.js_engine +
           [JS_OPTIMIZER, temp_file, 'minifyGlobals', 'noPrintMetadata'] +
@@ -249,10 +258,9 @@ class Minifier(object):
     self.globs = json.loads(metadata)
 
     if self.symbols_file:
-      mapfile = open(self.symbols_file, 'w')
-      for key, value in self.globs.items():
-        mapfile.write(value + ':' + key + '\n')
-      mapfile.close()
+      with open(self.symbols_file, 'w') as f:
+        for key, value in self.globs.items():
+          f.write(value + ':' + key + '\n')
       print('wrote symbol map file to', self.symbols_file, file=sys.stderr)
 
     return code.replace('13371337', '0.0')
@@ -280,15 +288,14 @@ def run_on_chunk(command):
       while os.path.exists(saved): saved = 'input' + str(int(saved.replace('input', '').replace('.txt', ''))+1) + '.txt'
       print('running js optimizer command', ' '.join([c if c != filename else saved for c in command]), file=sys.stderr)
       shutil.copyfile(filename, os.path.join(shared.get_emscripten_temp_dir(), saved))
-    if shared.EM_BUILD_VERBOSE_LEVEL >= 3: print('run_on_chunk: ' + str(command), file=sys.stderr)
+    if shared.EM_BUILD_VERBOSE >= 3: print('run_on_chunk: ' + str(command), file=sys.stderr)
     proc = shared.run_process(command, stdout=subprocess.PIPE)
     output = proc.stdout
     assert proc.returncode == 0, 'Error in optimizer (return code ' + str(proc.returncode) + '): ' + output
     assert len(output) and not output.startswith('Assertion failed'), 'Error in optimizer: ' + output
     filename = temp_files.get(os.path.basename(filename) + '.jo.js').name
-    f = open(filename, 'w')
-    f.write(output)
-    f.close()
+    with open(filename, 'w') as f:
+      f.write(output)
     if DEBUG and not shared.WINDOWS: print('.', file=sys.stderr) # Skip debug progress indicator on Windows, since it doesn't buffer well with multiple threads printing to console.
     return filename
   except KeyboardInterrupt:
@@ -406,7 +413,7 @@ EMSCRIPTEN_FUNCS();
   with ToolchainProfiler.profile_block('js_optimizer.split_to_chunks'):
     # if we are making source maps, we want our debug numbering to start from the
     # top of the file, so avoid breaking the JS into chunks
-    cores = 1 if source_map else int(os.environ.get('EMCC_CORES') or multiprocessing.cpu_count())
+    cores = 1 if source_map else shared.Building.get_num_cores()
 
     if not just_split:
       intended_num_chunks = int(round(cores * NUM_CHUNKS_PER_CORE))
@@ -429,10 +436,9 @@ EMSCRIPTEN_FUNCS();
       with ToolchainProfiler.profile_block('js_optimizer.write_chunks'):
         def write_chunk(chunk, i):
           temp_file = temp_files.get('.jsfunc_%d.js' % i).name
-          f = open(temp_file, 'w')
-          f.write(chunk)
-          f.write(serialized_extra_info)
-          f.close()
+          with open(temp_file, 'w') as f:
+            f.write(chunk)
+            f.write(serialized_extra_info)
           return temp_file
         filenames = [write_chunk(chunks[i], i) for i in range(len(chunks))]
     else:
@@ -475,20 +481,18 @@ EMSCRIPTEN_FUNCS();
       cl_sep = 'wakaUnknownBefore(); var asm=wakaUnknownAfter(global,env,buffer)\n'
 
       with temp_files.get_file('.cl.js') as cle:
-        c = open(cle, 'w')
         pre_1, pre_2 = pre.split(start_asm)
         post_1, post_2 = post.split(end_asm)
-        c.write(pre_1)
-        c.write(cl_sep)
-        c.write(post_2)
-        c.close()
+        with open(cle, 'w') as f:
+          f.write(pre_1)
+          f.write(cl_sep)
+          f.write(post_2)
         cld = cle
         if split_memory:
           if DEBUG: print('running splitMemory on shell code', file=sys.stderr)
           cld = run_on_chunk(js_engine + [JS_OPTIMIZER, cld, 'splitMemoryShell'])
-          f = open(cld, 'a')
-          f.write(suffix_marker)
-          f.close()
+          with open(cld, 'a') as f:
+            f.write(suffix_marker)
         if closure:
           if DEBUG: print('running closure on shell code', file=sys.stderr)
           cld = shared.Building.closure_compiler(cld, pretty='minifyWhitespace' not in passes)
@@ -579,5 +583,5 @@ if __name__ == '__main__':
     shutil.copyfile(out, sys.argv[1] + '.jsopt.js')
   except Exception as e:
     ToolchainProfiler.record_process_exit(1)
-    raise e
+    raise
   ToolchainProfiler.record_process_exit(0)
